﻿/*
 * rScripting Engine for .NET Framework
 * Website: http://rScript.Codeplex.com
 * 
 * Microsoft Public License (Ms-PL)
 * This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
 * 1. Definitions
 *    The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
 *    A "contribution" is the original software, or any additions or changes to the software.
 *    A "contributor" is any person that distributes its contribution under this license.
 *    "Licensed patents" are a contributor's patent claims that read directly on its contribution.
 * 2. Grant of Rights
 *   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
 *   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
 * 3. Conditions and Limitations
 *   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
 *   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
 *   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
 *   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
 *   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */

using System;

using rScripting;
using rScripting.Compilers;
using rScripting.LateBinding;

namespace RuntimeExample
{
    class Program
    {
        //Instance of the rScript Compile Engine that is used throughout the example application.
        private static CompileEngine _CompileEngine;

        static void Main(string[] args)
        {
            //Setup our compiler engine
            _CompileEngine = new CompileEngine();
            
            /*
             * Supported Compilers:
             *      - C#
             *      - CSharp
             *      - VB
             *      - VisualBasic
             *      - JScript
             * Note:
             * The C# and CSharp options reference the same compiler.
             * The same is applied to the VisualBasic and VB compilers.
             */
            _CompileEngine.Compiler = "C#";
            _CompileEngine.ScriptExtension = ".cs";

            //Add a reference to the windows forms dll as our script uses it for showing the MessageBox 
            _CompileEngine.AddAssemblyReference("System.Windows.Forms.dll");

            //Example of how to compile your raw source code
            CompileSource();

            //Example of how to compile your script files individually.
            CompileScriptFile();

            //Example of how to compile a directory filled with script files.
            CompileScriptRepository();

            //Create a Script Factory. This contains a collection of all scripts compiled, if multiple are compiled.
            ScriptFactory factory = new ScriptFactory(_CompileEngine.CompiledAssembly);

            //Get a reference to our compiled script.
            ScriptObject script = factory.GetScript("Wheel");

            //Write out the size of the Wheel, by accessing it's properties.
            Console.WriteLine("The size of the wheel is: " + script.GetField("Size"));
            Console.WriteLine("The width of the wheel is: " + script.GetProperty("Width"));

            //Invoke the Scripts Method to display a message box.
            script.InvokeMethod("GetTireBrand");
        }

        static void CompileSource()
        {
            //Construct an array of strings with raw source code.
            String[] sourceCode = new String[] {"using System; \nnamespace SampleSource {\n" + 
                "public class SampleClass {" +
                "public Int32 Size = 20; " +
                "public double Width" +
                "{" +
                    "get" +
                    "{" +
                      "return 8.5;" +
                    "}" +
                "}" + 
                "public void GetTireBrand() {" + 
                    "System.Windows.Forms.MessageBox.Show(\"Super Tire Co.\");" +
                 "}" + 
                 "}" + 
                 "}"
            };

            //Compile the source code
            Boolean isCompiled = _CompileEngine.Compile(sourceCode);

            //Check for any compiler errors.
            if (!isCompiled)
            {
                Console.WriteLine(_CompileEngine.Errors);
            }
        }

        static void CompileScriptFile()
        {
            //The script file that we will compile
            System.IO.FileInfo file = new System.IO.FileInfo(@"Scripts\Wheel.cs");

            //Compile our source script file. Script engine will perform file existance checking prior to compiling.
            Boolean isCompiled = _CompileEngine.Compile(file);

            //Check for any compiler errors.
            if (!isCompiled)
            {
                Console.WriteLine(_CompileEngine.Errors);
            }
        }

        static void CompileScriptRepository()
        {
            //The directory that contains a collection of scripts.
            String scriptRepository = "Scripts";

            //Compile all scripts within that directory.
            Boolean isCompiled = _CompileEngine.Compile(scriptRepository);

            //Check for any compiler errors.
            if (!isCompiled)
            {
                Console.WriteLine(_CompileEngine.Errors);
            }
        }
    }
}